// Gmail Manager
// By Todd Long <longfocus@gmail.com>
// http://www.longfocus.com/firefox/gmanager/

const nsIGmailAccount = Components.interfaces.nsIGmailAccount;
const nsIPrefService = Components.interfaces.nsIPrefService;
const nsIPasswordManagerInternal = Components.interfaces.nsIPasswordManagerInternal;
const nsIPasswordManager = Components.interfaces.nsIPasswordManager;

const GMANAGER_BRANCH = "gmanager.";
const GMANAGER_PASS_SITE = "gmanager.gmail.account";

function GmailManager() {};
GmailManager.prototype = {
  _firstTime: true,
  _accounts: null,
  _branch: null,
  
  getNumAccounts: function()
  {
    return this._accounts ? this._accounts.length : 0;
  },
  
  getAccount: function(aNum)
  {
    return this._accounts ? this._accounts[aNum] : null;
  },
  
  getAccountIndex: function(aUsername)
  {
    var index = -1;
    
    for (var i = 0; i < this._accounts.length && index == -1; i++)
      if (this._accounts[i].username == aUsername)
        index = i;
    
    return index;
  },
  
  load: function()
  {
    if (this._firstTime)
    {
      this._firstTime = false;
      this._branch = Components.classes["@mozilla.org/preferences-service;1"].getService(nsIPrefService).getBranch(GMANAGER_BRANCH);
      this._loadAccounts();
    }
  },
  
  addAccount: function(aUsername, aPassword, aStorePassword)
  {
    if (!this._exists(aUsername) && this._validate(aUsername))
    {
      this._accounts[this._accounts.length] = this._createAccount(aUsername, aPassword, aStorePassword, "GMAIL");
      
      if (aStorePassword)
        this._storePassword(aUsername, aPassword);
      
      this._saveAccounts();
      
      return true;
    }
    
    return false;
  },
  
  removeAccount: function(aUsername)
  {
    var index = this.getAccountIndex(aUsername);
    
    if (this.findPassword(aUsername) != "")
      this._removeStoredPassword(aUsername);
    
    this._accounts.splice(index, 1);
    this._saveAccounts();
  },
  
  findPassword: function(aUsername)
  {
    try {
      var host = {value:""};
      var user = {value:""};
      var password = {value:""};
      var passService = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManagerInternal);
      passService.findPasswordEntry(GMANAGER_PASS_SITE, aUsername, "", host, user, password);
    } catch(e) {}
    
    return password.value;
  },
  
  savePassword: function(aUsername, aPassword, aRemember)
  {
    var index = this.getAccountIndex(aUsername);
    
    this._accounts[index].password = aPassword;
    this._accounts[index].remember = aRemember;
    
    this._removeStoredPassword(aUsername);
    
    if (aRemember)
      this._storePassword(aUsername, aPassword);
  },
  
  _loadAccounts: function()
  {
    var myAccounts = this._branch.getCharPref("accounts").split(",");
    
    this._accounts = new Array();
    
    for (var i = 0; i < myAccounts.length; i++)
    {
      var username = myAccounts[i];
      
      if (this._validate(username))
      {
        // Checks for an existing password
        var password = this.findPassword(username);
        var remember = (password != "");
        
        this._accounts[this._accounts.length] = 
          this._createAccount(username, password, remember, "GMAIL")
      }
    }
  },
  
  _saveAccounts: function()
  {
    var myAccounts = new Array();
    
    for (var i = 0; i < this._accounts.length; i++)
      myAccounts[i] = this._accounts[i].username;
    
    // Saves the accounts list
    this._branch.setCharPref("accounts", myAccounts.join(","));
  },
  
  _createAccount: function(aUsername, aPassword, aRemember, aProtocol)
  {
    // Create an account object
    var account = Components.classes["@longfocus.com/gmanager/account/gmail;1"].createInstance(nsIGmailAccount);
    
    // Set account information
    account.username = aUsername;
    account.password = aPassword;
    account.remember = aRemember;
    //account.protocol = aProtocol;
    
    return account;
  },
  
  _storePassword: function(aUsername, aPassword)
  {
    try {
      var passMan = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
      
      if (aPassword != "")
        passMan.addUser(GMANAGER_PASS_SITE, aUsername, aPassword);
    } catch (e) {}
  },
  
  _removeStoredPassword: function(aUsername)
  {
    try {
      var passMan = Components.classes["@mozilla.org/passwordmanager;1"].getService(nsIPasswordManager);
      
      passMan.removeUser(GMANAGER_PASS_SITE, aUsername);
    } catch (e) {}
  },
  
  _exists: function(aUsername)
  {
    var exists = false;
    
    for (var i = 0; i < this._accounts.length && !exists; i++)
      if (this._accounts[i].username == aUsername)
        exists = true;
    
    return exists;
  },
  
  _validate: function(aUsername)
  {
    // Straight from Gmail:
    // Sorry, only letters (a-z), numbers (0-9), and periods (.) are allowed.
    
    if (aUsername == "" || aUsername.search(/[^a-zA-Z0-9.]/) > -1)
      return false;
    else
      return true;
  },
  
  QueryInterface: function (iid)
  {
    if (iid.equals(Components.interfaces.nsIGmailManager) ||
        iid.equals(Components.interfaces.nsISupports)) return this;
    
    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
    return null;
  }
}

var myModule = {
  firstTime: true,
  
  myCID: Components.ID("{CD555771-7A24-47fb-BA88-DCA115449E77}"),
  myDesc: "Webmail accounts manager",
  myProgID: "@longfocus.com/gmanager/manager;1",
  myFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      
      return (new GmailManager()).QueryInterface(iid);
    }
  },
  
  registerSelf: function (compMgr, fileSpec, location, type)
  {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.myCID, this.myDesc, this.myProgID, fileSpec, location, type);
  },
 
  getClassObject: function (compMgr, cid, iid)
  {
    if (!cid.equals(this.myCID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return this.myFactory;
  },
  
  canUnload: function(compMgr) { return true; }
};

function NSGetModule(compMgr, fileSpec) { return myModule; }
